"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupByChain = exports.getAllowanceViaMulticall = exports.setApproval = exports.getApproved = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const ethers_1 = require("ethers");
const ChainsService_1 = __importDefault(require("../services/ChainsService"));
const types_1 = require("../types");
const errors_1 = require("../utils/errors");
const multicall_1 = require("../utils/multicall");
const getApproved = async (signer, tokenAddress, contractAddress) => {
    const signerAddress = await signer.getAddress();
    const erc20 = new ethers_1.Contract(tokenAddress, types_1.ERC20_ABI, signer);
    try {
        const approved = await erc20.allowance(signerAddress, contractAddress);
        return new bignumber_js_1.default(approved.toString());
    }
    catch (e) {
        return new bignumber_js_1.default(0);
    }
};
exports.getApproved = getApproved;
const setApproval = async (signer, tokenAddress, contractAddress, amount, returnPopulatedTransaction) => {
    const erc20 = new ethers_1.Contract(tokenAddress, types_1.ERC20_ABI, signer);
    if (returnPopulatedTransaction) {
        return erc20.populateTransaction.approve(contractAddress, amount);
    }
    const transactionRequest = await erc20.populateTransaction.approve(contractAddress, amount);
    try {
        const estimatedGasLimit = await signer.estimateGas(transactionRequest);
        if (estimatedGasLimit) {
            const formattedGasLimit = ethers_1.ethers.BigNumber.from(`${(BigInt(estimatedGasLimit.toString()) * 125n) / 100n}`);
            transactionRequest.gasLimit = formattedGasLimit;
        }
    }
    catch (error) { }
    return signer.sendTransaction(transactionRequest);
};
exports.setApproval = setApproval;
const getAllowanceViaMulticall = async (signer, chainId, tokenData) => {
    const chainsService = ChainsService_1.default.getInstance();
    const chain = await chainsService.getChainById(chainId);
    if (!chain.multicallAddress) {
        throw new errors_1.ServerError(`No multicall address configured for chainId ${chainId}.`);
    }
    const ownerAddress = await signer.getAddress();
    const calls = [];
    tokenData.map(({ token, approvalAddress }) => {
        calls.push({
            address: token.address,
            name: 'allowance',
            params: [ownerAddress, approvalAddress],
        });
    });
    const result = await (0, multicall_1.fetchDataUsingMulticall)(calls, types_1.ERC20_ABI, chainId, chain.multicallAddress);
    if (!result.length) {
        throw new errors_1.ServerError(`Couldn't load allowance from chainId ${chainId} using multicall.`);
    }
    const parsedResult = result.map(({ data }) => ({
        approvalAmount: data ?? new bignumber_js_1.default(0),
    }));
    return tokenData.map(({ token, approvalAddress }, i) => ({
        token,
        approvalAddress,
        approvedAmount: parsedResult[i].approvalAmount,
    }));
};
exports.getAllowanceViaMulticall = getAllowanceViaMulticall;
const groupByChain = (tokenDataList) => {
    // group by chain
    const tokenDataByChain = {};
    tokenDataList.forEach((tokenData) => {
        if (!tokenDataByChain[tokenData.token.chainId]) {
            tokenDataByChain[tokenData.token.chainId] = [];
        }
        tokenDataByChain[tokenData.token.chainId].push(tokenData);
    });
    return tokenDataByChain;
};
exports.groupByChain = groupByChain;
