"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const DefaultExecutionSettings = {
    /* eslint-disable-next-line @typescript-eslint/no-empty-function */
    updateRouteHook: () => { },
    switchChainHook: () => Promise.resolve(undefined),
    acceptExchangeRateUpdateHook: () => Promise.resolve(undefined),
    infiniteApproval: false,
    executeInBackground: false,
};
class ConfigService {
    constructor() {
        this.resolveSetupPromise = undefined;
        /**
         * This call immediately returns the current config. It does not make sure that all chain data is already loaded
         * Use this if you need access to basic information like API urls or settings
         */
        this.getConfig = () => {
            return this.config;
        };
        /**
         * This call waits for all setup promises to be done.
         * Use this if you need access to chain data (RPCs or multicalls)
         */
        this.getConfigAsync = async () => {
            await this.setupPromise;
            return this.config;
        };
        this.updateConfig = (configUpdate) => {
            // API
            this.config.apiUrl = configUpdate.apiUrl || this.config.apiUrl;
            // RPCS
            this.config.rpcs = Object.assign(this.config.rpcs, configUpdate.rpcs);
            // MULTICALL
            this.config.multicallAddresses = Object.assign(this.config.multicallAddresses, configUpdate.multicallAddresses);
            // SETTINGS
            this.config.defaultExecutionSettings = Object.assign(this.config.defaultExecutionSettings, configUpdate.defaultExecutionSettings);
            // OPTIONS
            this.config.defaultRouteOptions = Object.assign(this.config.defaultRouteOptions, configUpdate.defaultRouteOptions);
            this.config.apiKey = configUpdate.apiKey || this.config.apiKey;
            this.config.userId = configUpdate.userId || this.config.userId;
            this.config.integrator = configUpdate.integrator || this.config.integrator;
            this.config.defaultRouteOptions.integrator =
                configUpdate.integrator ||
                    configUpdate.defaultRouteOptions?.integrator ||
                    this.config.integrator;
            this.config.widgetVersion =
                configUpdate.widgetVersion || this.config.widgetVersion;
            this.config.multisigConfig =
                configUpdate.multisigConfig || this.config.multisigConfig;
            return this.config;
        };
        this.updateChains = (chains) => {
            for (const chain of chains) {
                const chainId = chain.id;
                // set RPCs if they were not configured by the user before
                if (!this.config.rpcs[chainId]?.length) {
                    this.config.rpcs[chainId] = chain.metamask.rpcUrls;
                }
                // set multicall addresses if they exist and were not configured by the user before
                if (chain.multicallAddress && !this.config.multicallAddresses[chainId]) {
                    this.config.multicallAddresses[chainId] = chain.multicallAddress;
                }
            }
            this.resolveSetupPromise?.();
            return this.config;
        };
        this.config = ConfigService.getDefaultConfig();
        this.setupPromise = new Promise((resolve) => {
            this.resolveSetupPromise = resolve;
        });
    }
    static chainIdToObject(val) {
        const result = {};
        const values = Object.values(types_1.ChainId);
        values.forEach((chainId) => {
            if (typeof chainId !== 'string') {
                result[chainId] = val ? JSON.parse(JSON.stringify(val)) : val;
            }
        });
        return result;
    }
    static getInstance() {
        if (!this.instance) {
            this.instance = new ConfigService();
        }
        return this.instance;
    }
}
ConfigService.getDefaultConfig = () => {
    return {
        apiUrl: 'https://li.quest/v1',
        rpcs: ConfigService.chainIdToObject([]),
        multicallAddresses: ConfigService.chainIdToObject(undefined),
        defaultExecutionSettings: DefaultExecutionSettings,
        defaultRouteOptions: {
            integrator: 'lifi-sdk',
        },
        integrator: 'lifi-sdk',
    };
};
exports.default = ConfigService;
