import BigNumber from 'bignumber.js';
import { constants } from 'ethers';
import { getRpcProvider } from '../connectors';
export const sleep = (mills) => {
    return new Promise((resolve) => {
        setTimeout(resolve, mills);
    });
};
export const personalizeStep = async (signer, step) => {
    if (step.action.toAddress && step.action.fromAddress) {
        return step;
    }
    const address = await signer.getAddress();
    const fromAddress = step.action.fromAddress || address;
    const toAddress = step.action.toAddress || address;
    return {
        ...step,
        action: {
            ...step.action,
            fromAddress,
            toAddress,
        },
    };
};
export const splitListIntoChunks = (list, chunkSize) => list.reduce((resultList, item, index) => {
    const chunkIndex = Math.floor(index / chunkSize);
    if (!resultList[chunkIndex]) {
        resultList[chunkIndex] = []; // start a new chunk
    }
    resultList[chunkIndex].push(item);
    return resultList;
}, []);
export const formatTokenAmountOnly = (token, amount) => {
    if (!amount) {
        return '0.0';
    }
    let floated;
    if (typeof amount === 'string') {
        if (amount === '0') {
            return '0.0';
        }
        floated = new BigNumber(amount).shiftedBy(-token.decimals);
    }
    else {
        floated = amount;
        if (floated.isZero()) {
            return '0.0';
        }
    }
    // show at least 4 decimal places and at least two non-zero digests
    let decimalPlaces = 3;
    while (floated.lt(1 / 10 ** decimalPlaces)) {
        decimalPlaces++;
    }
    return floated.toFixed(decimalPlaces + 1, 1);
};
/**
 * Repeatedly calls a given asynchronous function until it resolves with a value
 * @param toRepeat The function that should be repeated
 * @param timeout The timeout in milliseconds between retries, defaults to 5000
 * @returns The result of the toRepeat function
 */
export const repeatUntilDone = async (toRepeat, timeout = 5000) => {
    let result;
    while (!result) {
        result = await toRepeat();
        if (!result) {
            await sleep(timeout);
        }
    }
    return result;
};
/**
 * Loads a transaction receipt using the rpc for the given chain id
 * @param chainId The chain id where the transaction should be loaded from
 * @param txHash The hash of the transaction
 * @returns TransactionReceipt
 */
export const loadTransactionReceipt = async (chainId, txHash) => {
    const rpc = await getRpcProvider(chainId);
    const tx = await rpc.getTransaction(txHash);
    return tx.wait();
};
export const isZeroAddress = (address) => {
    if (address === constants.AddressZero ||
        address === '0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee') {
        return true;
    }
    return false;
};
export const isNativeTokenAddress = (address) => {
    if (address === constants.AddressZero ||
        address === '0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee' ||
        // CELO native token
        address === '0x471ece3750da237f93b8e339c536989b8978a438') {
        return true;
    }
    return false;
};
